#include "stdafx.h"
#include <string.h>
#include <ctype.h>   
#include <math.h>
#include "utility.h"

static int ctoup(int c);
int static ctolow(int c);

static char	buff2[40];


///////////////////////////////////////////////////////////////////////////
//	Desc: Fills a control with data elements (Combo box or List box)
//	params: ptr to window containg the control,control ID,list of data elements (null terminated),
//			list of dwords if needed
//	returns: none ( id Data is null no data is added via setitemdata
///////////////////////////////////////////////////////////////////////////
void FillControl(CWnd* pParent,UINT nControlID,char** Items,DWORD* Data)
{                          
	BOOL	IsCombo,IsList;                           
	int	y,nIndex;             
	char TheName[30];         
	CComboBox* pCombo;
	CListBox* pList;

	ASSERT(pParent != NULL);
	ASSERT(Items != NULL);
    CWnd* pControlWnd=pParent->GetDlgItem(nControlID);
	if(pControlWnd == NULL)
	{
		TRACE("Control ID Invalid or parent window pointer invalid\n");
		return ;
	}	
	::GetClassName(pControlWnd->m_hWnd, TheName, sizeof(TheName));
	IsCombo=lstrcmpi("combobox", TheName) == 0 ? TRUE:FALSE;
	IsList=lstrcmpi("listbox", TheName) == 0 ? TRUE:FALSE;
	if(IsList==FALSE && IsCombo==FALSE)
	{
		TRACE("Control is not a combo or list box\n");
		return ;                                                        
	}	
	if(IsCombo)                     
	{
		pCombo=(CComboBox*)pParent->GetDlgItem(nControlID);
		TRACE("Filling a combo box\n");
		pCombo->ResetContent();
		for(y=0;y !=-1;++y)                   
		{
			if(Items[y]==NULL)
				return;
			nIndex=pCombo->AddString(Items[y]);					
			if(Data != NULL)
				pCombo->SetItemData(nIndex,Data[y] );				
		}		
	}	
	else
	{
		pList=(CListBox*)pParent->GetDlgItem(nControlID);	
		TRACE("Filling a list box\n");
		pList->ResetContent();
		for(y=0;y !=-1;++y)                   
		{
			if(Items[y]==NULL)
				return;
			nIndex=pList->AddString(Items[y]);					
			if(Data != NULL)
				pList->SetItemData(nIndex,Data[y] );				
		}		
	}	
}
///////////////////////////////////////////////////////////////////////////
//	Desc:	Makes a control visible or invisible
//	params: ptr to win containg control,control ID FALSE= not visible TRUE=visible
//	returns:none
///////////////////////////////////////////////////////////////////////////
void SetVisibleState(CWnd *pParent,int nID,BOOL mode)
{                    
	ASSERT(pParent != NULL);
	CWnd *ptr=pParent->GetDlgItem(nID);
	ASSERT(ptr != NULL);
	if(ptr==NULL)
		return;
	if(mode==FALSE)
	{
		ptr->ShowWindow(SW_HIDE);
	}			
	else
	{
		ptr->ShowWindow(SW_SHOW);
	}			
}

///////////////////////////////////////////////////////////////////////////
//	Desc:	enab,es or disables (greys out ) a control
//	params:	ptr to win containg control,control ID FALSE= greyed TRUE=Normal
//	returns:
///////////////////////////////////////////////////////////////////////////
void SetControlState(CWnd *pParent,int nID,BOOL mode)
{                    
	ASSERT(pParent != NULL);
	CWnd *ptr=pParent->GetDlgItem(nID);
	ASSERT(ptr != NULL);
	if(ptr==NULL)
		return;       
	ptr->EnableWindow(mode);		
}


void  DDX_LBString(CDataExchange* pDX, int nIDC, LPSTR value,int maxLen)
{
	HWND hWndCtrl = pDX->PrepareCtrl(nIDC);
	if (pDX->m_bSaveAndValidate)
	{
		int nIndex = (int)::SendMessage(hWndCtrl, LB_GETCURSEL, 0, 0L);
		if (nIndex != -1)
		{
			int nLen = (int)::SendMessage(hWndCtrl, LB_GETTEXTLEN, nIndex, 0L);
			::SendMessage(hWndCtrl, LB_GETTEXT, nIndex,
					(LPARAM)(LPSTR)value);
		}
		else
		{
			// no selection
			strcpy(value,"");
		}
	}
	else
	{
		// set current selection based on data string
		if (::SendMessage(hWndCtrl, LB_SELECTSTRING, (WPARAM)-1,
		  (LPARAM)(LPCSTR)value) == LB_ERR)
		{
			// no selection match
			TRACE0("Warning: no listbox item selected\n");
		}
	}
}

void DDX_CBString(CDataExchange* pDX, int nIDC, LPSTR value,int maxLen)
{
	HWND hWndCtrl = pDX->PrepareCtrl(nIDC);
	if (pDX->m_bSaveAndValidate)
	{
		// just get current edit item text (or drop list static)
		int nLen = ::GetWindowTextLength(hWndCtrl);
		if (nLen != -1)
		{
			// get known length
			::GetWindowText(hWndCtrl, value, nLen+1 < maxLen ? nLen+1:maxLen);
		}
		else
		{
			::GetWindowText(hWndCtrl, value, maxLen);
		}
	}
	else
	{
		// set current selection based on model string
		if (::SendMessage(hWndCtrl, CB_SELECTSTRING, (WPARAM)-1,
			(LPARAM)(LPCSTR)value) == CB_ERR)
		{
			// just set the edit text (will be ignored if DROPDOWNLIST)
			::SetWindowText(hWndCtrl, value);
		}
	}
}

///////////////////////////////////////////////////////////////////////////
//	Desc:	data exchange function for a C (null terminated) type string
//	params: PDX*, control id,the string,the length ( not counting 0 byte)
//	returns:none
///////////////////////////////////////////////////////////////////////////
void DDX_Text(CDataExchange *pDX,int nID,LPSTR str,int nLen)
{                      
	char 	buff[50]; 
	pDX->PrepareEditCtrl(nID); 
	
	CEdit *ptr=(CEdit *)pDX->m_pDlgWnd->GetDlgItem(nID);
	if(pDX->m_bSaveAndValidate)
	{
		if(ptr->LineLength() > nLen)
		{                             
			sprintf(buff,"Max Length is %d",nLen);
			pDX->m_pDlgWnd->MessageBox(buff,"",MB_OK);
			pDX->Fail();                          
		}			
		pDX->m_pDlgWnd->GetDlgItem(nID)->GetWindowText(str,nLen+1);
	}
	else
	{                                            
		ptr->LimitText(nLen);
		pDX->m_pDlgWnd->GetDlgItem(nID)->SetWindowText(str);	
	}
}	

////////////////////////////////////////////////////////////////////
//  PARAMETERS:    double to be converted,length of output string
//                 direction of padding "r" or "l" or "R" or "L"
//  DESCRIPTION:   formats a double to have commas for dollar output
//  RETURNS:       the converted string (static)
///////////////////////////////////////////////////////////////////
static char*	AdjustNeg(char *p,BOOL Neg)
{
	static	char	buff[61];
	static	char	buff2[61];
	int	y;	
	for(y=0;y < (signed)strlen(p);++y)
		if(p[y] !=' ')
			break;
	if(Neg)
	{
		strcpy(buff,"-");
		strcat(buff,p+y);
		sprintf(buff2,"%s",buff);
		return buff2;
	}
	return p;
}
	
	
////////////////////////////////////////////////////////////////////
//  PARAMETERS:    double to be converted
//  DESCRIPTION:   formats a double to have commas for dollar output
//  RETURNS:       the converted string
///////////////////////////////////////////////////////////////////
LPSTR DollarFormat(double amt)
{   
	int	y,len,cnt;
	static	char	buff[61];
	static	char	buff2[61];                
	BOOL	Neg=FALSE;
	
	memset( (void *)buff2,'\0',sizeof(buff2));	
	
	if(amt < 0)
	{
		Neg=TRUE;
		amt*=-1;
	}
	
	sprintf(buff,"%.2f",amt);
	if(amt < 1000.00)
	{
		return AdjustNeg(buff,Neg);			
	}		
	
	char	*p,*c,*e;
	p=strtok(buff,".");
	len=strlen(p);
	if(p)
	{
	 	c=&buff2[55];
	 	cnt=0;
	 	for(y=len;y > 0;--y)
	 	{
	 		*c--=buff[y-1];
	 		++cnt;
	 		if(cnt==3 && (y-1) > 0)
	 		{
	 			cnt=0;
				*c--=',';
			}
		}
		e=&buff2[56];
		*e++='.';
		p=strtok(NULL,".");
		strncpy(e,p,2);		 			
		for(y=0;y < 61;++y)
		{
			if(buff2[y]!='\0')
			{
				c=&buff2[y];
				break;
			}
		}					
		strcpy(buff,c);
		return AdjustNeg(buff,Neg);			
	}
	return AdjustNeg(buff,Neg);				
}


double  Calc_Apr(int numpay,double monp,double amt)
{
	double diff,d,bi,li,apr;
	double bia[3],a[3],lia[3];
	register int j;
	register int k1;

	bi=29;
	do{
		li=bi / 1200;
		bia[1]=bi;
		bia[2]=bi + 0.1;
		for(j=1;j < 3;j++)
		{
			lia[j]=bia[j] / 1200;
			d=1;
			for(k1=1;k1 < numpay;++k1)
				d=d+(1 / pow( ( 1 +lia[j] ), ( double ) k1 ) );
			a[j]=monp * ( d / ( 1+lia[j] ) );
		}
		apr=bi + 0.1 * ( ( amt-a[1] ) / ( a[2]-a[1] ) );
		diff=fabs ( bi-apr );
		if( diff >= 0.01 )  bi=apr;
	}while ( diff > 0.01 );
	return( apr );
}




/*======================================================================*/
/* returns a number rounded to the requested number of decimal places   */ 
/* for example sround(1.348665,2) returns 1.35				*/
/* caveat sround(1.5,0) returns 2.0 for insurance purposes		*/
/*======================================================================*/
double  sround(double Num,int pre)
{                    
	double res1,res2,diff;
	
	if(pre==0)
	{
		res1=floor(Num);
		diff=Num-res1;
		if(diff > .49)
		{
			res1+=1;
			return res1;
		}
		return res1;
	}
	else
	{
		res1=Num*pow(10.0,(double)pre);
		res2=floor(res1);
		diff=res1-res2;
		if(diff > .49)
			res2+=1;
		res2=res2/pow(10.0,(double)pre);
		return res2;
	}
}
/*======================================================================*/
/* truncate a number to a decimal place. for example			*/
/* Trunc(1.34567,2) returns 1.34					*/
/*======================================================================*/
double  Trunc(double Num,int pre)
{                    
	double res1;
	
	if(pre==0)
	{
		res1=floor(Num);
		return res1;
	}
	else
	{
		res1=Num*pow(10.0,(double)pre);
		res1=floor(res1);
		res1=res1/pow(10.0,(double)pre);
		return res1;
	}
}
/*======================================================================*/
/* will return a true value if number n(integer) is is set a(int array) */ 
/*======================================================================*/
			
BOOL  isset(int *a,int n)
{
	int *p;

	p=a;
	while(*p!=-1)
	     if(n==*p++)
		  return TRUE;
	return FALSE;
}


BOOL  isset(double *a,double n)
{
	double *p;

	p=a;
	while(*p!=-1)
	     if(n==*p++)
		  return TRUE;
	return FALSE;
}

LPSTR  FixName(LPSTR ndx)
{
        int                             flag=1;
        unsigned 				        i;

        for(i=0; i < strlen(ndx);i++)
        {
                if(isspace(ndx[i])) flag=1;
                if(ispunct(ndx[i]) && ndx[i]!=39) flag=1;
                if(isdigit(ndx[i]) && flag)
                        flag=0;
                if((ndx[i]=='M' || ndx[i]=='m') && flag)
                        if(ndx[i+1]=='c' || ndx[i+1]=='C')
                                if(isalpha(ndx[i+2]))
                                        ndx[i+2]=ctoup(ndx[i+2]);
                if(isalpha(ndx[i]) && flag)
                {
                        ndx[i]=ctoup(ndx[i]);
                        flag=0;
                }
        }
        return ndx;
}

static int ctoup(int c)
{
        if (c >= 'a' && c <= 'z')
        return (c - 32);
           else
             return(c);                     /* convert to upper case */
     }

int static ctolow(int c)
{
        if (c >= 'A' && c <='Z')
        return(c+32);
                else
                 return(c);
}

